@@ -1,5 +1,5 @@
 from .transfer import transfer
-from .contact import add_contact, show_all_contact, show_contact_by_name
+from .contact import add_contact, show_all_contact, show_contact_by_name, delete_contact, edit_contact
 from .type import Action, ActionData
 from .authorize import show_authorized_tokens, authorize_token
 
@@ -11,5 +11,7 @@ __all__ = [
     'authorize_token',
     'show_authorized_tokens'
     'Action',
-    'ActionData'
+    'ActionData',
+    'delete_contact',
+    'edit_contact'
 ]
\ No newline at end of file
