@@ -7,24 +7,97 @@ from actions_lib.utils.contact_tool import get_contact_address
 # Constants
 TOKEN_MAPPING = {
     "ethereum": {
-        "usdc": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
+        "eth": {
+            "address": None,
+            "decimal": 18
+        },
+        "usdc": {
+            "address": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
+            "decimal": 6
+        },
+        "usdt": {
+            "address": "0xdAC17F958D2ee523a2206206994597C13D831ec7",
+            "decimal": 6
+        },
+        "bnb": {
+            "address": "0xB8c77482e45F1F44dE1745F52C74426C631bDD52",
+            "decimal": 18
+        },
+        "dai": {
+            "address": "0x6B175474E89094C44Da98b954EedeAC495271d0F",
+            "decimal": 18
+        },
+        "uni": {
+            "address": "0x1f9840a85d5aF5bf1D1762F925BDADdC4201F984",
+            "decimal": 18
+        },
+        "btc": {
+            "address": "0x2260FAC5E5542a773Aa44fBCfeDf7C193bc2C599",
+            "decimal": 8
+        },
+        "ens": {
+            "address": "0xC18360217D8F7Ab5e7c516566761Ea12Ce7F9D72",
+            "decimal": 18
+        }
     },
     "eth": {
-        "usdc": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
-    },
-    "bsc": {
-        "usdc": "0x8ac76a51cc950d9822d68b83fe1ad97b32cd580d",
+        "eth": {
+            "address": None,
+            "decimal": 18
+        },
+        "usdc": {
+            "address": "0xd10519Aa03FE7Ffb95189B52B74F94Fb33E2C88a",
+            "decimal": 6
+        },
+        "usdt": {
+            "address": "0xdAC17F958D2ee523a2206206994597C13D831ec7",
+            "decimal": 6
+        },
+        "bnb": {
+            "address": "0xB8c77482e45F1F44dE1745F52C74426C631bDD52",
+            "decimal": 18
+        },
+        "dai": {
+            "address": "0x6B175474E89094C44Da98b954EedeAC495271d0F",
+            "decimal": 18
+        },
+        "uni": {
+            "address": "0x1f9840a85d5aF5bf1D1762F925BDADdC4201F984",
+            "decimal": 18
+        },
+        "btc": {
+            "address": "0x2260FAC5E5542a773Aa44fBCfeDf7C193bc2C599",
+            "decimal": 8
+        },
+        "ens": {
+            "address": "0xC18360217D8F7Ab5e7c516566761Ea12Ce7F9D72",
+            "decimal": 18
+        }
     },
     "base": {
-        "usdc": "0x45b58118a5374dccf7fd6f3f66c66278ab7473d9",
+        "eth": {
+            "address": None,
+            "decimal": 18
+        },
+        "usdc": {
+            "address": "0x45b58118a5374dccf7fd6f3f66c66278ab7473d9",
+            "decimal": 6
+        },
+        "btc": {
+            "address": "0xcbB7C0000aB88B473b1f5aFd9ef808440eed33Bf",
+            "decimal": 8
+        },
+        "aero": {
+            "address": "0x940181a94A35A4569E4529A3CDfB74e38FD98631",
+            "decimal": 18
+        }
     },
 }
 
 CHAIN_ID_MAPPING = {
     'base': '0x14a34',
     'ethereum': '0xaa36a7',
-    'eth': '0xaa36a7',
-    'bsc': '0x38'
+    'eth': '0xaa36a7'
 }
 
 # ABIs
@@ -60,6 +133,16 @@ def get_web3(providers, chain):
         providers[chain]['w3'] = Web3(HTTPProvider(providers[chain]['rpc']))
     return providers[chain]['w3']
 
+def convert_to_wei(amount, chain, token):
+    decimal = TOKEN_MAPPING[chain][token.lower()]['decimal']
+    amount = float(amount) * (10 ** decimal)
+    return int(amount)
+
+def convert_to_value(amount, chain, token):
+    decimal = TOKEN_MAPPING[chain][token.lower()]['decimal']
+    amount = amount / (10 ** decimal)
+    return float(amount)
+
 def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', chain: str = "base", token: str = "usdc", **kwargs: Any) -> Dict[str, Any]:
     print(f"receiver: {receiver}, amount: {amount}, run_mode: {run_mode}, chain: {chain}, token: {token}, kwargs: {kwargs}, step: {step}")
     
@@ -81,7 +164,11 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
     w3 = get_web3(providers, chain_raw)
     eth_w3 = get_web3(providers, 'eth')
     token_raw = token.lower()
-    token_address = w3.to_checksum_address(TOKEN_MAPPING[chain_raw][token_raw])
+    token_address = TOKEN_MAPPING[chain_raw][token_raw]['address']
+    if token_address is not None:
+        token_address = w3.to_checksum_address(token_address)
+    print(f"token address: {token_address}", flush=True)
+    print(f"token address: {token_address is None}", flush=True)
     
     print(f"w3: {w3}", flush=True)
 
@@ -91,43 +178,52 @@ def transfer(receiver: str, amount: float, step: any, run_mode: str = 'auto', ch
         return handle_manual_transfer(w3, eth_w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw)
 
 def handle_auto_transfer(w3: Web3, eth_w3, executor, receiver, amount, token_address, spender_address, sender_address, control_contract, private_key, redis_client, chain_raw, token_raw, blockchain_explore, executor_balance):
-    token_contract = w3.eth.contract(address=token_address, abi=ALLOWANCE_ABI)
-    allowance = token_contract.functions.allowance(Web3.to_checksum_address(executor), spender_address).call()
-    allowance_normalized = Web3.from_wei(allowance, 'mwei')
-    print(f"allowance_normalized: {float(allowance_normalized)}")
-    
-    if float(allowance_normalized) < float(amount):
-        return handle_insufficient_allowance(w3, eth_w3, executor, receiver, amount, spender_address, token_address, chain_raw, token_raw, redis_client)
+    if token_raw != 'usdc':
+        token = token_raw.upper()
+        if token_raw == 'btc':
+            if chain_raw == 'base':
+                token = 'cbBTC'
+            else:
+                token = 'WBTC'
+        return {'result': {"code": 500, "content": f"Transfers of the current currency({token}) are not supported in automatic mode."}, 'action': None, 'next_action': None}
+    else:
+        token_contract = w3.eth.contract(address=token_address, abi=ALLOWANCE_ABI)
+        allowance = token_contract.functions.allowance(Web3.to_checksum_address(executor), spender_address).call()
+        allowance_normalized = convert_to_value(allowance, chain_raw, token_raw)
+        print(f"allowance_normalized: {float(allowance_normalized)}")
+        
+        if float(allowance_normalized) < float(amount):
+            return handle_insufficient_allowance(w3, eth_w3, executor, receiver, amount, spender_address, token_address, chain_raw, token_raw, redis_client)
 
-    code, receiver_address = get_contact_address(redis_client, executor, receiver, eth_w3)
-    if code != 200:
-        return {'result': {"content": f"Cannot find {receiver} on contact", "code": 420}, 'action': None, 'next_action': None}
+        code, receiver_address = get_contact_address(redis_client, executor, receiver, eth_w3)
+        if code != 200:
+            return {'result': {"content": f"Cannot find {receiver} on contact", "code": 420}, 'action': None, 'next_action': None}
 
-    amount_to_transfer = w3.to_wei(float(amount), 'mwei')
-    to = w3.to_checksum_address(receiver_address)
-    executor = w3.to_checksum_address(executor)
+        amount_to_transfer = convert_to_wei(float(amount), chain_raw, token_raw)
+        to = w3.to_checksum_address(receiver_address)
+        executor = w3.to_checksum_address(executor)
 
-    print(f"to: {to}, executor: {executor}")
-    
-    try:
-        can_send, tx_hash = maybe_send_transaction(w3, control_contract, sender_address, token_address, executor, to, amount_to_transfer, private_key, chain_raw, executor_balance)
-        if can_send:
-            tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)
-            status = 200 if tx_receipt['status'] == 1 else 500
-            info = {
-                "content": generate_markdown_content(tx_receipt['status'], tx_hash.hex(), blockchain_explore),
-                "tx_hash": tx_hash.hex(),
-                "chain": chain_raw,
-                "status": tx_receipt['status'],
-                "code": status
-            }
-            return {'result': info, 'action': None, 'next_action': None}
-        else:
-            return {'result': {"code": 500, "content": "Insufficient balance. Estimated gas cost exceed account balance"}, 'action': None, 'next_action': None}
-    except Exception as e:
-        return handle_transaction_error(e, control_contract, w3)
+        print(f"to: {to}, executor: {executor}")
+        
+        try:
+            can_send, tx_hash = maybe_send_transaction(w3, control_contract, sender_address, token_address, executor, to, amount_to_transfer, private_key, chain_raw, executor_balance)
+            if can_send:
+                tx_receipt = w3.eth.wait_for_transaction_receipt(tx_hash)
+                status = 200 if tx_receipt['status'] == 1 else 500
+                info = {
+                    "content": generate_markdown_content(tx_receipt['status'], tx_hash.hex(), blockchain_explore),
+                    "tx_hash": tx_hash.hex(),
+                    "chain": chain_raw,
+                    "status": tx_receipt['status'],
+                    "code": status
+                }
+                return {'result': info, 'action': None, 'next_action': None}
+            else:
+                return {'result': {"code": 500, "content": "Insufficient balance. Estimated gas cost exceed account balance"}, 'action': None, 'next_action': None}
+        except Exception as e:
+            return handle_transaction_error(e, control_contract, w3)
 
-def handle_manual_transfer(w3, eth_w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw):
+def handle_manual_transfer(w3: Web3, eth_w3, executor, receiver, amount, token_address, redis_client, chain_raw, token_raw):
     active_chain_key = f"user:{executor}:active_chain"
     active_chain = redis_client.get(active_chain_key) or "base"
     print(f"active_chain_key: {active_chain_key}, active_chain: {active_chain}", flush=True)
@@ -142,22 +238,41 @@ def handle_manual_transfer(w3, eth_w3, executor, receiver, amount, token_address
     if code != 200:
         return {'result': {"content": f"Cannot find {receiver} on contact", "code": 420}, 'action': None, 'next_action': None}
 
-    amount_to_transfer = w3.to_wei(float(amount), 'mwei')
+    amount_to_transfer = convert_to_wei(float(amount), chain_raw, token_raw)
     to = w3.to_checksum_address(receiver_address)
     params = {
-        'recipient': to,
+        'from': executor,
+        'to': to,
         'amount': amount_to_transfer,
-        'chainId': CHAIN_ID_MAPPING[chain_raw],
-        'contract': token_address,
-        'func': 'transfer',
-        'abi': ERC20_TRANSFER_ABI
+        'token': token_address,
+        'chain': param_chain_id
     }
+    
+    if token_address is None:
+        # token_address = TOKEN_MAPPING[chain_raw]['usdc']['address']
+        params = {
+            'to': to,
+            'amount': amount_to_transfer,
+            'chainId': CHAIN_ID_MAPPING[chain_raw],
+            'contract': None,
+            'func': 'transferETH',
+            'abi': None
+        }
+    else:
+        params = {
+            'recipient': to,
+            'amount': amount_to_transfer,
+            'chainId': CHAIN_ID_MAPPING[chain_raw],
+            'contract': token_address,
+            'func': 'transfer',
+            'abi': ERC20_TRANSFER_ABI
+        }
     action_data = ActionData(func='transfer', group='transfer', params=params)
     action = Action(msg=None, type='wallet', data=action_data.__dict__)
     return {'result': {"code": 200, "content": "Manual transfer parameters are constructed"}, 'action': action.__dict__, 'next_action': None}
 
 def handle_insufficient_allowance(w3, eth_w3, executor, receiver, amount, spender_address, token_address, chain_raw, token_raw, redis_client):
-    amount_to_transfer = w3.to_wei(float(amount), 'mwei')
+    amount_to_transfer = convert_to_wei(float(amount), chain_raw, token_raw)
     action_params = {
         "func": "approve",
         "chainId": CHAIN_ID_MAPPING[chain_raw],
@@ -221,8 +336,8 @@ def maybe_send_transaction(w3: Web3, control_contract, sender_address, token, ex
     gas_cost = gas_estimate * new_gas_price
     
     print(f"gas_estimate: {gas_estimate}, gas_cost: {gas_cost}, executor_balance: {executor_balance}")
-    # if executor_balance[chain] < gas_cost:
-    #     return False, 0x0
+    if executor_balance[chain] < gas_cost:
+        return False, 0x0
     
     signed_tx = w3.eth.account.sign_transaction(tx, private_key)
     return True, w3.eth.send_raw_transaction(signed_tx.rawTransaction)
